Package: pubmedR
Title: Gathering Metadata About Publications, Grants, Clinical Trials
        from 'PubMed' Database
Version: 1.0.2
Authors@R: 
    person(given = "Massimo",
           family = "Aria",
           role = c("aut", "cre"),
           email = "massimo.aria@gmail.com",
           comment = c(ORCID = "0000-0002-8517-9411"))
Description: A set of tools to extract bibliographic content from 'PubMed' database
    using 'NCBI' REST API <https://www.ncbi.nlm.nih.gov/home/develop/api/>.
    It includes functions to search, download, and convert 'PubMed' bibliographic
    records into data frames compatible with the 'bibliometrix' package.
    Features include programmatic query building, batch downloading by PMID,
    citation enrichment via 'NCBI' E-Link, and robust error handling with
    automatic retry logic.
License: GPL-3
URL: https://github.com/massimoaria/pubmedR
BugReports: https://github.com/massimoaria/pubmedR/issues
Encoding: UTF-8
Imports: rentrez, XML
Suggests: bibliometrix, knitr, rmarkdown, testthat (>= 3.0.0), withr
Config/testthat/edition: 3
RoxygenNote: 7.3.3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-04 18:14:49 UTC; massimoaria
Author: Massimo Aria [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8517-9411>)
Maintainer: Massimo Aria <massimo.aria@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-04 21:10:32 UTC
Built: R 4.6.0; ; 2026-05-04 22:08:34 UTC; unix
