# Generated by roxygen2: do not edit by hand

export(pmApi2df)
export(pmApiRequest)
export(pmCitedBy)
export(pmCollect)
export(pmEnrichCitations)
export(pmExtractReferences)
export(pmFetchById)
export(pmQueryBuild)
export(pmQueryTotalCount)
export(pmReferences)
import(XML)
import(rentrez)
